from ml.data.data import File_Queue
from ml.neural_network.layer import FlatteningLayer, FullyConnectedLayer
from ml.neural_network.neural_network import NeuralNetwork
from ml.neural_network.train import launch_training_task
import tensorflow as tf

def get_records(path):
    with open(path) as f:
        record_list = f.readlines()
    record_list = [record.strip() for record in record_list]
    return record_list

identity_f = lambda x: x

def cost_f(y, y_hat):
    return tf.reduce_mean(tf.nn.softmax_cross_entropy_with_logits(logits=y, labels=y_hat))

def accuracy_f(y, y_hat):
    correct_predictions_vector = tf.equal(tf.argmax(y, 1), tf.argmax(y_hat, 1))
    accuracy = tf.reduce_mean(tf.cast(correct_predictions_vector, tf.float32))
    return accuracy

gradient_f = tf.train.AdamOptimizer(1e-4).minimize

def prediction_f(y):
    return tf.argmax(y, 1)

def make_preprocess_function(label_to_filter):
    def preprocess(features):
        inp = tf.concat([features['mean_rgb'],  features['mean_audio']], 0)
        labels = tf.sparse_to_indicator(features["labels"], 4716)
        labels.set_shape([4716])
        single_label = labels[label_to_filter]
        single_label = tf.one_hot(indices=tf.cast(single_label, tf.int32), depth=2)
        single_label = tf.cast(single_label, tf.float32)
        return inp, single_label
    return preprocess

def make_conv_network(inputs_dim, labels_dim, input_queue):
    layer_1 = FullyConnectedLayer(name='layer_1', activation_f=tf.nn.relu)
    final_layer = FullyConnectedLayer(name='final_layer', activation_f=identity_f)
    layers = [layer_1, final_layer]
    layers_dims = [(1024,), (2,)]
    network = NeuralNetwork(inputs_dim, labels_dim, layers, layers_dims, cost_f, accuracy_f,
                            gradient_f, prediction_f, input_queue)
    network.make_computation_graph()
    return network

if __name__ == "__main__":
    feature_map = {'labels': tf.VarLenFeature(dtype=tf.int64),
                   'video_id': tf.FixedLenFeature(shape=[], dtype=tf.string, default_value=None),
                   'mean_rgb': tf.FixedLenFeature(shape=[1024], dtype=tf.float32, default_value=None),
                   'mean_audio': tf.FixedLenFeature(shape=[128], dtype=tf.float32, default_value=None)}
    inputs_dim = [1152]
    labels_dim = [2]
    preprocess = make_preprocess_function(0)
    record_list = ['gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train--.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train-0.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train-1.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train-2.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train-3.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train-4.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train-5.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train-6.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train-7.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train-8.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train-9.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train-A.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train-B.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train-C.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train-D.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train-E.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train-F.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train-G.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train-H.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train-I.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train-J.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train-K.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train-L.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train-M.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train-N.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train-O.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train-P.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train-Q.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train-R.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train-S.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train-T.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train-U.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train-V.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train-W.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train-X.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train-Y.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train-Z.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train-_.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train-a.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train-b.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train-c.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train-d.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train-e.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train-f.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train-g.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train-h.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train-i.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train-j.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train-k.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train-l.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train-m.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train-n.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train-o.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train-p.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train-q.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train-r.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train-s.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train-t.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train-u.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train-v.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train-w.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train-x.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train-y.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train-z.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train0-.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train00.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train01.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train02.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train03.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train04.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train05.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train06.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train07.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train08.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train09.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train0A.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train0B.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train0C.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train0D.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train0E.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train0F.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train0G.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train0H.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train0I.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train0J.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train0K.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train0L.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train0M.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train0N.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train0O.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train0P.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train0Q.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train0R.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train0S.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train0T.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train0U.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train0V.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train0W.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train0X.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train0Y.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train0Z.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train0_.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train0a.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train0b.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train0c.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train0d.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train0e.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train0f.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train0g.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train0h.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train0i.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train0j.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train0k.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train0l.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train0m.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train0n.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train0o.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train0p.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train0q.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train0r.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train0s.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train0t.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train0u.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train0v.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train0w.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train0x.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train0y.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train0z.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train1-.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train10.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train11.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train12.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train13.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train14.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train15.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train16.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train17.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train18.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train19.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train1A.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train1B.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train1C.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train1D.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train1E.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train1F.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train1G.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train1H.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train1I.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train1J.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train1K.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train1L.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train1M.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train1N.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train1O.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train1P.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train1Q.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train1R.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train1S.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train1T.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train1U.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train1V.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train1W.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train1X.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train1Y.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train1Z.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train1_.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train1a.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train1b.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train1c.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train1d.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train1e.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train1f.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train1g.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train1h.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train1i.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train1j.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train1k.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train1l.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train1m.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train1n.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train1o.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train1p.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train1q.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train1r.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train1s.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train1t.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train1u.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train1v.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train1w.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train1x.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train1y.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train1z.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train2-.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train20.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train21.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train22.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train23.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train24.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train25.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train26.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train27.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train28.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train29.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train2A.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train2B.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train2C.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train2D.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train2E.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train2F.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train2G.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train2H.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train2I.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train2J.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train2K.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train2L.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train2M.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train2N.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train2O.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train2P.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train2Q.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train2R.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train2S.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train2T.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train2U.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train2V.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train2W.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train2X.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train2Y.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train2Z.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train2_.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train2a.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train2b.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train2c.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train2d.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train2e.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train2f.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train2g.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train2h.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train2i.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train2j.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train2k.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train2l.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train2m.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train2n.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train2o.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train2p.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train2q.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train2r.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train2s.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train2t.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train2u.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train2v.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train2w.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train2x.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train2y.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train2z.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train3-.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train30.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train31.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train32.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train33.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train34.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train35.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train36.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train37.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train38.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train39.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train3A.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train3B.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train3C.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train3D.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train3E.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train3F.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train3G.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train3H.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train3I.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train3J.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train3K.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train3L.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train3M.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train3N.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train3O.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train3P.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train3Q.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train3R.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train3S.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train3T.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train3U.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train3V.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train3W.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train3X.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train3Y.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train3Z.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train3_.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train3a.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train3b.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train3c.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train3d.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train3e.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train3f.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train3g.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train3h.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train3i.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train3j.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train3k.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train3l.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train3m.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train3n.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train3o.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train3p.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train3q.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train3r.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train3s.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train3t.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train3u.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train3v.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train3w.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train3x.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train3y.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train3z.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train4-.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train40.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train41.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train42.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train43.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train44.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train45.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train46.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train47.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train48.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train49.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train4A.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train4B.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train4C.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train4D.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train4E.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train4F.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train4G.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train4H.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train4I.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train4J.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train4K.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train4L.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train4M.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train4N.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train4O.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train4P.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train4Q.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train4R.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train4S.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train4T.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train4U.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train4V.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train4W.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train4X.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train4Y.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train4Z.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train4_.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train4a.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train4b.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train4c.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train4d.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train4e.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train4f.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train4g.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train4h.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train4i.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train4j.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train4k.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train4l.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train4m.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train4n.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train4o.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train4p.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train4q.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train4r.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train4s.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train4t.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train4u.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train4v.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train4w.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train4x.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train4y.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train4z.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train5-.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train50.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train51.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train52.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train53.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train54.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train55.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train56.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train57.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train58.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train59.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train5A.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train5B.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train5C.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train5D.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train5E.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train5F.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train5G.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train5H.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train5I.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train5J.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train5K.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train5L.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train5M.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train5N.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train5O.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train5P.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train5Q.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train5R.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train5S.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train5T.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train5U.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train5V.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train5W.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train5X.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train5Y.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train5Z.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train5_.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train5a.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train5b.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train5c.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train5d.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train5e.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train5f.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train5g.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train5h.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train5i.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train5j.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train5k.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train5l.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train5m.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train5n.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train5o.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train5p.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train5q.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train5r.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train5s.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train5t.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train5u.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train5v.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train5w.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train5x.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train5y.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train5z.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train6-.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train60.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train61.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train62.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train63.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train64.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train65.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train66.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train67.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train68.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train69.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train6A.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train6B.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train6C.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train6D.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train6E.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train6F.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train6G.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train6H.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train6I.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train6J.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train6K.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train6L.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train6M.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train6N.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train6O.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train6P.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train6Q.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train6R.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train6S.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train6T.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train6U.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train6V.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train6W.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train6X.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train6Y.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train6Z.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train6_.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train6a.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train6b.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train6c.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train6d.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train6e.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train6f.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train6g.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train6h.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train6i.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train6j.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train6k.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train6l.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train6m.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train6n.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train6o.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train6p.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train6q.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train6r.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train6s.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train6t.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train6u.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train6v.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train6w.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train6x.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train6y.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train6z.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train7-.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train70.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train71.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train72.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train73.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train74.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train75.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train76.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train77.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train78.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train79.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train7A.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train7B.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train7C.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train7D.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train7E.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train7F.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train7G.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train7H.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train7I.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train7J.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train7K.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train7L.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train7M.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train7N.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train7O.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train7P.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train7Q.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train7R.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train7S.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train7T.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train7U.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train7V.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train7W.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train7X.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train7Y.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train7Z.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train7_.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train7a.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train7b.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train7c.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train7d.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train7e.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train7f.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train7g.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train7h.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train7i.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train7j.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train7k.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train7l.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train7m.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train7n.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train7o.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train7p.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train7q.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train7r.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train7s.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train7t.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train7u.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train7v.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train7w.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train7x.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train7y.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train7z.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train8-.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train80.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train81.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train82.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train83.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train84.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train85.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train86.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train87.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train88.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train89.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train8A.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train8B.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train8C.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train8D.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train8E.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train8F.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train8G.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train8H.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train8I.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train8J.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train8K.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train8L.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train8M.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train8N.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train8O.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train8P.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train8Q.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train8R.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train8S.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train8T.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train8U.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train8V.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train8W.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train8X.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train8Y.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train8Z.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train8_.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train8a.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train8b.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train8c.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train8d.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train8e.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train8f.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train8g.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train8h.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train8i.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train8j.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train8k.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train8l.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train8m.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train8n.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train8o.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train8p.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train8q.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train8r.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train8s.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train8t.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train8u.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train8v.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train8w.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train8x.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train8y.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train8z.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train9-.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train90.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train91.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train92.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train93.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train94.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train95.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train96.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train97.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train98.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train99.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train9A.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train9B.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train9C.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train9D.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train9E.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train9F.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train9G.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train9H.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train9I.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train9J.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train9K.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train9L.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train9M.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train9N.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train9O.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train9P.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train9Q.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train9R.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train9S.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train9T.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train9U.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train9V.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train9W.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train9X.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train9Y.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train9Z.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train9_.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train9a.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train9b.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train9c.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train9d.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train9e.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train9f.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train9g.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train9h.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train9i.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train9j.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train9k.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train9l.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train9m.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train9n.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train9o.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train9p.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train9q.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train9r.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train9s.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train9t.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train9u.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train9v.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train9w.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train9x.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train9y.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train9z.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainA-.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainA0.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainA1.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainA2.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainA3.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainA4.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainA5.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainA6.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainA7.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainA8.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainA9.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainAA.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainAB.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainAC.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainAD.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainAE.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainAF.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainAG.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainAH.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainAI.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainAJ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainAK.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainAL.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainAM.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainAN.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainAO.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainAP.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainAQ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainAR.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainAS.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainAT.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainAU.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainAV.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainAW.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainAX.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainAY.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainAZ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainA_.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainAa.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainAb.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainAc.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainAd.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainAe.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainAf.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainAg.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainAh.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainAi.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainAj.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainAk.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainAl.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainAm.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainAn.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainAo.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainAp.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainAq.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainAr.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainAs.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainAt.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainAu.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainAv.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainAw.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainAx.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainAy.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainAz.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainB-.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainB0.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainB1.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainB2.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainB3.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainB4.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainB5.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainB6.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainB7.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainB8.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainB9.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainBA.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainBB.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainBC.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainBD.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainBE.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainBF.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainBG.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainBH.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainBI.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainBJ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainBK.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainBL.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainBM.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainBN.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainBO.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainBP.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainBQ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainBR.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainBS.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainBT.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainBU.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainBV.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainBW.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainBX.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainBY.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainBZ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainB_.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainBa.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainBb.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainBc.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainBd.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainBe.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainBf.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainBg.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainBh.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainBi.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainBj.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainBk.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainBl.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainBm.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainBn.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainBo.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainBp.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainBq.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainBr.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainBs.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainBt.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainBu.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainBv.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainBw.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainBx.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainBy.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainBz.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainC-.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainC0.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainC1.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainC2.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainC3.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainC4.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainC5.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainC6.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainC7.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainC8.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainC9.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainCA.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainCB.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainCC.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainCD.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainCE.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainCF.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainCG.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainCH.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainCI.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainCJ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainCK.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainCL.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainCM.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainCN.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainCO.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainCP.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainCQ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainCR.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainCS.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainCT.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainCU.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainCV.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainCW.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainCX.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainCY.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainCZ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainC_.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainCa.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainCb.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainCc.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainCd.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainCe.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainCf.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainCg.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainCh.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainCi.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainCj.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainCk.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainCl.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainCm.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainCn.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainCo.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainCp.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainCq.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainCr.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainCs.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainCt.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainCu.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainCv.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainCw.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainCx.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainCy.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainCz.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainD-.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainD0.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainD1.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainD2.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainD3.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainD4.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainD5.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainD6.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainD7.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainD8.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainD9.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainDA.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainDB.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainDC.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainDD.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainDE.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainDF.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainDG.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainDH.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainDI.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainDJ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainDK.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainDL.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainDM.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainDN.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainDO.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainDP.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainDQ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainDR.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainDS.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainDT.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainDU.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainDV.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainDW.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainDX.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainDY.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainDZ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainD_.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainDa.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainDb.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainDc.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainDd.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainDe.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainDf.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainDg.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainDh.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainDi.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainDj.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainDk.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainDl.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainDm.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainDn.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainDo.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainDp.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainDq.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainDr.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainDs.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainDt.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainDu.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainDv.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainDw.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainDx.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainDy.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainDz.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainE-.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainE0.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainE1.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainE2.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainE3.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainE4.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainE5.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainE6.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainE7.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainE8.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainE9.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainEA.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainEB.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainEC.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainED.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainEE.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainEF.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainEG.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainEH.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainEI.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainEJ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainEK.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainEL.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainEM.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainEN.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainEO.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainEP.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainEQ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainER.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainES.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainET.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainEU.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainEV.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainEW.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainEX.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainEY.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainEZ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainE_.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainEa.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainEb.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainEc.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainEd.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainEe.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainEf.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainEg.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainEh.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainEi.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainEj.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainEk.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainEl.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainEm.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainEn.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainEo.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainEp.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainEq.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainEr.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainEs.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainEt.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainEu.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainEv.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainEw.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainEx.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainEy.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainEz.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainF-.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainF0.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainF1.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainF2.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainF3.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainF4.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainF5.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainF6.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainF7.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainF8.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainF9.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainFA.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainFB.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainFC.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainFD.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainFE.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainFF.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainFG.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainFH.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainFI.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainFJ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainFK.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainFL.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainFM.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainFN.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainFO.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainFP.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainFQ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainFR.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainFS.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainFT.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainFU.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainFV.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainFW.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainFX.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainFY.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainFZ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainF_.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainFa.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainFb.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainFc.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainFd.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainFe.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainFf.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainFg.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainFh.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainFi.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainFj.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainFk.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainFl.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainFm.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainFn.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainFo.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainFp.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainFq.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainFr.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainFs.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainFt.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainFu.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainFv.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainFw.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainFx.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainFy.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainFz.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainG-.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainG0.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainG1.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainG2.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainG3.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainG4.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainG5.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainG6.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainG7.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainG8.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainG9.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainGA.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainGB.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainGC.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainGD.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainGE.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainGF.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainGG.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainGH.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainGI.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainGJ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainGK.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainGL.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainGM.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainGN.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainGO.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainGP.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainGQ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainGR.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainGS.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainGT.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainGU.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainGV.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainGW.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainGX.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainGY.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainGZ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainG_.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainGa.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainGb.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainGc.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainGd.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainGe.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainGf.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainGg.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainGh.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainGi.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainGj.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainGk.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainGl.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainGm.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainGn.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainGo.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainGp.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainGq.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainGr.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainGs.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainGt.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainGu.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainGv.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainGw.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainGx.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainGy.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainGz.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainH-.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainH0.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainH1.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainH2.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainH3.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainH4.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainH5.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainH6.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainH7.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainH8.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainH9.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainHA.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainHB.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainHC.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainHD.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainHE.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainHF.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainHG.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainHH.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainHI.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainHJ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainHK.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainHL.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainHM.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainHN.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainHO.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainHP.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainHQ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainHR.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainHS.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainHT.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainHU.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainHV.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainHW.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainHX.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainHY.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainHZ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainH_.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainHa.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainHb.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainHc.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainHd.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainHe.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainHf.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainHg.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainHh.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainHi.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainHj.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainHk.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainHl.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainHm.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainHn.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainHo.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainHp.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainHq.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainHr.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainHs.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainHt.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainHu.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainHv.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainHw.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainHx.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainHy.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainHz.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainI-.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainI0.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainI1.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainI2.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainI3.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainI4.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainI5.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainI6.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainI7.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainI8.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainI9.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainIA.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainIB.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainIC.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainID.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainIE.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainIF.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainIG.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainIH.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainII.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainIJ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainIK.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainIL.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainIM.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainIN.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainIO.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainIP.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainIQ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainIR.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainIS.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainIT.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainIU.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainIV.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainIW.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainIX.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainIY.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainIZ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainI_.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainIa.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainIb.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainIc.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainId.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainIe.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainIf.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainIg.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainIh.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainIi.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainIj.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainIk.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainIl.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainIm.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainIn.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainIo.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainIp.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainIq.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainIr.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainIs.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainIt.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainIu.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainIv.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainIw.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainIx.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainIy.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainIz.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainJ-.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainJ0.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainJ1.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainJ2.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainJ3.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainJ4.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainJ5.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainJ6.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainJ7.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainJ8.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainJ9.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainJA.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainJB.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainJC.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainJD.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainJE.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainJF.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainJG.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainJH.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainJI.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainJJ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainJK.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainJL.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainJM.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainJN.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainJO.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainJP.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainJQ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainJR.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainJS.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainJT.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainJU.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainJV.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainJW.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainJX.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainJY.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainJZ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainJ_.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainJa.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainJb.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainJc.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainJd.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainJe.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainJf.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainJg.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainJh.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainJi.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainJj.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainJk.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainJl.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainJm.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainJn.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainJo.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainJp.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainJq.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainJr.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainJs.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainJt.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainJu.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainJv.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainJw.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainJx.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainJy.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainJz.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainK-.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainK0.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainK1.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainK2.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainK3.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainK4.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainK5.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainK6.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainK7.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainK8.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainK9.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainKA.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainKB.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainKC.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainKD.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainKE.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainKF.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainKG.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainKH.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainKI.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainKJ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainKK.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainKL.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainKM.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainKN.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainKO.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainKP.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainKQ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainKR.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainKS.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainKT.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainKU.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainKV.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainKW.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainKX.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainKY.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainKZ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainK_.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainKa.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainKb.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainKc.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainKd.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainKe.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainKf.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainKg.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainKh.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainKi.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainKj.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainKk.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainKl.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainKm.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainKn.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainKo.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainKp.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainKq.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainKr.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainKs.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainKt.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainKu.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainKv.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainKw.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainKx.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainKy.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainKz.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainL-.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainL0.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainL1.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainL2.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainL3.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainL4.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainL5.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainL6.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainL7.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainL8.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainL9.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainLA.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainLB.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainLC.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainLD.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainLE.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainLF.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainLG.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainLH.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainLI.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainLJ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainLK.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainLL.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainLM.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainLN.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainLO.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainLP.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainLQ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainLR.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainLS.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainLT.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainLU.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainLV.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainLW.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainLX.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainLY.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainLZ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainL_.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainLa.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainLb.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainLc.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainLd.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainLe.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainLf.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainLg.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainLh.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainLi.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainLj.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainLk.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainLl.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainLm.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainLn.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainLo.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainLp.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainLq.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainLr.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainLs.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainLt.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainLu.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainLv.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainLw.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainLx.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainLy.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainLz.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainM-.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainM0.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainM1.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainM2.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainM3.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainM4.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainM5.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainM6.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainM7.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainM8.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainM9.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainMA.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainMB.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainMC.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainMD.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainME.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainMF.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainMG.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainMH.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainMI.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainMJ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainMK.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainML.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainMM.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainMN.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainMO.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainMP.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainMQ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainMR.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainMS.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainMT.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainMU.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainMV.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainMW.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainMX.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainMY.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainMZ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainM_.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainMa.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainMb.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainMc.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainMd.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainMe.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainMf.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainMg.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainMh.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainMi.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainMj.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainMk.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainMl.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainMm.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainMn.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainMo.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainMp.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainMq.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainMr.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainMs.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainMt.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainMu.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainMv.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainMw.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainMx.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainMy.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainMz.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainN-.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainN0.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainN1.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainN2.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainN3.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainN4.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainN5.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainN6.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainN7.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainN8.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainN9.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainNA.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainNB.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainNC.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainND.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainNE.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainNF.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainNG.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainNH.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainNI.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainNJ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainNK.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainNL.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainNM.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainNN.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainNO.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainNP.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainNQ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainNR.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainNS.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainNT.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainNU.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainNV.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainNW.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainNX.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainNY.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainNZ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainN_.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainNa.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainNb.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainNc.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainNd.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainNe.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainNf.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainNg.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainNh.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainNi.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainNj.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainNk.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainNl.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainNm.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainNn.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainNo.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainNp.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainNq.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainNr.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainNs.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainNt.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainNu.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainNv.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainNw.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainNx.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainNy.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainNz.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainO-.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainO0.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainO1.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainO2.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainO3.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainO4.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainO5.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainO6.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainO7.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainO8.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainO9.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainOA.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainOB.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainOC.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainOD.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainOE.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainOF.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainOG.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainOH.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainOI.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainOJ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainOK.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainOL.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainOM.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainON.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainOO.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainOP.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainOQ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainOR.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainOS.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainOT.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainOU.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainOV.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainOW.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainOX.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainOY.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainOZ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainO_.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainOa.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainOb.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainOc.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainOd.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainOe.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainOf.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainOg.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainOh.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainOi.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainOj.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainOk.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainOl.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainOm.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainOn.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainOo.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainOp.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainOq.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainOr.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainOs.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainOt.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainOu.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainOv.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainOw.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainOx.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainOy.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainOz.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainP-.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainP0.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainP1.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainP2.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainP3.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainP4.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainP5.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainP6.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainP7.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainP8.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainP9.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainPA.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainPB.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainPC.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainPD.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainPE.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainPF.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainPG.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainPH.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainPI.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainPJ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainPK.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainPL.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainPM.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainPN.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainPO.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainPP.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainPQ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainPR.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainPS.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainPT.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainPU.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainPV.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainPW.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainPX.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainPY.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainPZ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainP_.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainPa.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainPb.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainPc.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainPd.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainPe.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainPf.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainPg.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainPh.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainPi.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainPj.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainPk.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainPl.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainPm.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainPn.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainPo.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainPp.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainPq.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainPr.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainPs.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainPt.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainPu.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainPv.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainPw.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainPx.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainPy.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainPz.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainQ-.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainQ0.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainQ1.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainQ2.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainQ3.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainQ4.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainQ5.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainQ6.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainQ7.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainQ8.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainQ9.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainQA.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainQB.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainQC.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainQD.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainQE.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainQF.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainQG.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainQH.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainQI.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainQJ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainQK.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainQL.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainQM.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainQN.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainQO.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainQP.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainQQ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainQR.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainQS.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainQT.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainQU.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainQV.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainQW.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainQX.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainQY.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainQZ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainQ_.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainQa.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainQb.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainQc.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainQd.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainQe.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainQf.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainQg.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainQh.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainQi.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainQj.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainQk.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainQl.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainQm.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainQn.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainQo.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainQp.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainQq.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainQr.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainQs.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainQt.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainQu.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainQv.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainQw.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainQx.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainQy.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainQz.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainR-.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainR0.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainR1.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainR2.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainR3.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainR4.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainR5.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainR6.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainR7.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainR8.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainR9.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainRA.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainRB.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainRC.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainRD.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainRE.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainRF.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainRG.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainRH.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainRI.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainRJ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainRK.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainRL.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainRM.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainRN.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainRO.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainRP.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainRQ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainRR.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainRS.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainRT.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainRU.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainRV.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainRW.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainRX.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainRY.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainRZ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainR_.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainRa.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainRb.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainRc.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainRd.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainRe.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainRf.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainRg.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainRh.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainRi.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainRj.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainRk.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainRl.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainRm.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainRn.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainRo.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainRp.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainRq.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainRr.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainRs.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainRt.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainRu.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainRv.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainRw.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainRx.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainRy.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainRz.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainS-.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainS0.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainS1.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainS2.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainS3.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainS4.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainS5.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainS6.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainS7.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainS8.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainS9.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainSA.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainSB.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainSC.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainSD.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainSE.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainSF.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainSG.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainSH.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainSI.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainSJ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainSK.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainSL.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainSM.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainSN.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainSO.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainSP.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainSQ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainSR.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainSS.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainST.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainSU.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainSV.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainSW.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainSX.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainSY.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainSZ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainS_.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainSa.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainSb.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainSc.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainSd.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainSe.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainSf.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainSg.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainSh.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainSi.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainSj.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainSk.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainSl.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainSm.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainSn.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainSo.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainSp.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainSq.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainSr.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainSs.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainSt.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainSu.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainSv.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainSw.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainSx.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainSy.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainSz.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainT-.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainT0.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainT1.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainT2.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainT3.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainT4.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainT5.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainT6.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainT7.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainT8.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainT9.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainTA.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainTB.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainTC.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainTD.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainTE.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainTF.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainTG.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainTH.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainTI.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainTJ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainTK.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainTL.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainTM.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainTN.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainTO.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainTP.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainTQ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainTR.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainTS.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainTT.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainTU.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainTV.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainTW.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainTX.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainTY.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainTZ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainT_.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainTa.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainTb.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainTc.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainTd.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainTe.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainTf.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainTg.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainTh.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainTi.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainTj.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainTk.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainTl.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainTm.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainTn.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainTo.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainTp.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainTq.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainTr.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainTs.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainTt.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainTu.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainTv.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainTw.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainTx.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainTy.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainTz.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainU-.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainU0.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainU1.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainU2.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainU3.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainU4.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainU5.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainU6.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainU7.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainU8.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainU9.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainUA.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainUB.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainUC.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainUD.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainUE.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainUF.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainUG.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainUH.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainUI.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainUJ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainUK.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainUL.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainUM.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainUN.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainUO.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainUP.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainUQ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainUR.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainUS.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainUT.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainUU.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainUV.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainUW.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainUX.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainUY.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainUZ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainU_.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainUa.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainUb.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainUc.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainUd.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainUe.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainUf.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainUg.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainUh.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainUi.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainUj.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainUk.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainUl.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainUm.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainUn.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainUo.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainUp.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainUq.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainUr.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainUs.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainUt.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainUu.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainUv.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainUw.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainUx.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainUy.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainUz.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainV-.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainV0.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainV1.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainV2.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainV3.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainV4.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainV5.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainV6.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainV7.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainV8.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainV9.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainVA.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainVB.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainVC.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainVD.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainVE.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainVF.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainVG.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainVH.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainVI.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainVJ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainVK.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainVL.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainVM.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainVN.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainVO.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainVP.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainVQ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainVR.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainVS.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainVT.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainVU.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainVV.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainVW.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainVX.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainVY.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainVZ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainV_.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainVa.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainVb.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainVc.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainVd.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainVe.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainVf.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainVg.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainVh.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainVi.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainVj.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainVk.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainVl.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainVm.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainVn.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainVo.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainVp.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainVq.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainVr.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainVs.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainVt.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainVu.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainVv.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainVw.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainVx.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainVy.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainVz.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainW-.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainW0.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainW1.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainW2.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainW3.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainW4.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainW5.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainW6.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainW7.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainW8.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainW9.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainWA.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainWB.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainWC.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainWD.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainWE.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainWF.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainWG.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainWH.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainWI.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainWJ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainWK.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainWL.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainWM.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainWN.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainWO.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainWP.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainWQ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainWR.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainWS.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainWT.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainWU.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainWV.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainWW.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainWX.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainWY.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainWZ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainW_.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainWa.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainWb.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainWc.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainWd.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainWe.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainWf.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainWg.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainWh.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainWi.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainWj.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainWk.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainWl.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainWm.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainWn.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainWo.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainWp.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainWq.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainWr.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainWs.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainWt.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainWu.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainWv.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainWw.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainWx.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainWy.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainWz.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainX-.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainX0.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainX1.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainX2.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainX3.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainX4.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainX5.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainX6.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainX7.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainX8.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainX9.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainXA.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainXB.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainXC.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainXD.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainXE.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainXF.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainXG.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainXH.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainXI.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainXJ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainXK.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainXL.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainXM.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainXN.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainXO.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainXP.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainXQ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainXR.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainXS.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainXT.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainXU.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainXV.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainXW.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainXX.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainXY.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainXZ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainX_.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainXa.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainXb.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainXc.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainXd.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainXe.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainXf.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainXg.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainXh.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainXi.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainXj.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainXk.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainXl.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainXm.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainXn.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainXo.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainXp.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainXq.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainXr.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainXs.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainXt.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainXu.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainXv.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainXw.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainXx.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainXy.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainXz.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainY-.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainY0.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainY1.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainY2.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainY3.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainY4.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainY5.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainY6.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainY7.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainY8.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainY9.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainYA.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainYB.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainYC.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainYD.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainYE.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainYF.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainYG.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainYH.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainYI.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainYJ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainYK.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainYL.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainYM.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainYN.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainYO.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainYP.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainYQ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainYR.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainYS.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainYT.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainYU.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainYV.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainYW.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainYX.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainYY.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainYZ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainY_.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainYa.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainYb.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainYc.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainYd.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainYe.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainYf.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainYg.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainYh.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainYi.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainYj.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainYk.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainYl.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainYm.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainYn.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainYo.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainYp.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainYq.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainYr.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainYs.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainYt.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainYu.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainYv.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainYw.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainYx.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainYy.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainYz.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainZ-.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainZ0.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainZ1.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainZ2.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainZ3.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainZ4.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainZ5.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainZ6.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainZ7.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainZ8.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainZ9.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainZA.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainZB.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainZC.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainZD.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainZE.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainZF.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainZG.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainZH.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainZI.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainZJ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainZK.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainZL.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainZM.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainZN.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainZO.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainZP.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainZQ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainZR.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainZS.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainZT.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainZU.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainZV.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainZW.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainZX.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainZY.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainZZ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainZ_.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainZa.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainZb.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainZc.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainZd.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainZe.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainZf.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainZg.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainZh.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainZi.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainZj.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainZk.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainZl.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainZm.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainZn.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainZo.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainZp.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainZq.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainZr.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainZs.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainZt.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainZu.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainZv.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainZw.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainZx.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainZy.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainZz.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train_-.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train_0.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train_1.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train_2.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train_3.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train_4.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train_5.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train_6.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train_7.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train_8.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train_9.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train_A.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train_B.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train_C.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train_D.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train_E.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train_F.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train_G.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train_H.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train_I.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train_J.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train_K.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train_L.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train_M.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train_N.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train_O.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train_P.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train_Q.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train_R.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train_S.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train_T.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train_U.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train_V.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train_W.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train_X.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train_Y.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train_Z.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train__.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train_a.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train_b.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train_c.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train_d.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train_e.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train_f.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train_g.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train_h.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train_i.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train_j.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train_k.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train_l.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train_m.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train_n.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train_o.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train_p.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train_q.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train_r.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train_s.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train_t.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train_u.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train_v.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train_w.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train_x.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train_y.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/train_z.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traina-.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traina0.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traina1.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traina2.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traina3.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traina4.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traina5.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traina6.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traina7.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traina8.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traina9.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainaA.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainaB.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainaC.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainaD.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainaE.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainaF.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainaG.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainaH.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainaI.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainaJ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainaK.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainaL.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainaM.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainaN.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainaO.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainaP.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainaQ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainaR.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainaS.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainaT.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainaU.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainaV.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainaW.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainaX.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainaY.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainaZ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traina_.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainaa.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainab.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainac.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainad.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainae.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainaf.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainag.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainah.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainai.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainaj.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainak.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainal.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainam.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainan.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainao.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainap.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainaq.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainar.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainas.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainat.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainau.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainav.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainaw.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainax.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainay.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainaz.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainb-.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainb0.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainb1.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainb2.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainb3.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainb4.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainb5.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainb6.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainb7.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainb8.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainb9.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainbA.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainbB.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainbC.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainbD.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainbE.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainbF.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainbG.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainbH.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainbI.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainbJ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainbK.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainbL.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainbM.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainbN.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainbO.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainbP.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainbQ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainbR.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainbS.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainbT.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainbU.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainbV.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainbW.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainbX.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainbY.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainbZ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainb_.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainba.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainbb.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainbc.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainbd.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainbe.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainbf.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainbg.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainbh.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainbi.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainbj.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainbk.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainbl.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainbm.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainbn.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainbo.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainbp.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainbq.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainbr.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainbs.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainbt.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainbu.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainbv.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainbw.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainbx.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainby.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainbz.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainc-.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainc0.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainc1.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainc2.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainc3.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainc4.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainc5.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainc6.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainc7.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainc8.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainc9.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traincA.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traincB.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traincC.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traincD.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traincE.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traincF.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traincG.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traincH.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traincI.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traincJ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traincK.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traincL.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traincM.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traincN.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traincO.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traincP.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traincQ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traincR.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traincS.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traincT.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traincU.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traincV.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traincW.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traincX.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traincY.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traincZ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainc_.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainca.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traincb.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traincc.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traincd.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traince.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traincf.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traincg.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainch.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainci.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traincj.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainck.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traincl.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traincm.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traincn.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainco.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traincp.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traincq.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traincr.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traincs.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainct.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traincu.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traincv.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traincw.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traincx.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traincy.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traincz.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traind-.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traind0.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traind1.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traind2.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traind3.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traind4.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traind5.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traind6.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traind7.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traind8.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traind9.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traindA.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traindB.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traindC.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traindD.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traindE.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traindF.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traindG.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traindH.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traindI.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traindJ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traindK.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traindL.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traindM.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traindN.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traindO.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traindP.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traindQ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traindR.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traindS.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traindT.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traindU.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traindV.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traindW.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traindX.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traindY.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traindZ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traind_.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainda.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traindb.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traindc.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traindd.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainde.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traindf.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traindg.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traindh.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traindi.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traindj.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traindk.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traindl.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traindm.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traindn.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traindo.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traindp.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traindq.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traindr.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainds.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traindt.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traindu.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traindv.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traindw.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traindx.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traindy.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traindz.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traine-.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traine0.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traine1.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traine2.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traine3.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traine4.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traine5.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traine6.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traine7.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traine8.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traine9.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traineA.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traineB.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traineC.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traineD.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traineE.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traineF.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traineG.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traineH.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traineI.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traineJ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traineK.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traineL.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traineM.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traineN.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traineO.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traineP.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traineQ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traineR.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traineS.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traineT.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traineU.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traineV.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traineW.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traineX.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traineY.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traineZ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traine_.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainea.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traineb.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainec.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trained.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainee.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainef.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traineg.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traineh.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainei.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainej.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainek.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainel.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainem.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainen.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traineo.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainep.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traineq.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainer.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traines.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainet.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traineu.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainev.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainew.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainex.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainey.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainez.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainf-.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainf0.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainf1.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainf2.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainf3.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainf4.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainf5.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainf6.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainf7.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainf8.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainf9.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainfA.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainfB.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainfC.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainfD.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainfE.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainfF.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainfG.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainfH.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainfI.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainfJ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainfK.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainfL.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainfM.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainfN.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainfO.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainfP.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainfQ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainfR.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainfS.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainfT.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainfU.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainfV.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainfW.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainfX.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainfY.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainfZ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainf_.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainfa.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainfb.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainfc.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainfd.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainfe.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainff.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainfg.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainfh.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainfi.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainfj.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainfk.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainfl.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainfm.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainfn.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainfo.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainfp.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainfq.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainfr.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainfs.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainft.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainfu.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainfv.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainfw.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainfx.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainfy.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainfz.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traing-.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traing0.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traing1.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traing2.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traing3.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traing4.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traing5.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traing6.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traing7.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traing8.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traing9.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traingA.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traingB.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traingC.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traingD.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traingE.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traingF.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traingG.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traingH.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traingI.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traingJ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traingK.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traingL.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traingM.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traingN.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traingO.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traingP.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traingQ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traingR.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traingS.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traingT.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traingU.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traingV.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traingW.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traingX.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traingY.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traingZ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traing_.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainga.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traingb.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traingc.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traingd.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainge.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traingf.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traingg.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traingh.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traingi.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traingj.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traingk.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traingl.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traingm.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traingn.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traingo.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traingp.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traingq.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traingr.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traings.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traingt.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traingu.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traingv.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traingw.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traingx.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traingy.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traingz.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainh-.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainh0.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainh1.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainh2.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainh3.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainh4.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainh5.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainh6.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainh7.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainh8.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainh9.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainhA.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainhB.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainhC.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainhD.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainhE.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainhF.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainhG.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainhH.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainhI.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainhJ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainhK.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainhL.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainhM.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainhN.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainhO.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainhP.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainhQ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainhR.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainhS.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainhT.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainhU.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainhV.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainhW.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainhX.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainhY.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainhZ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainh_.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainha.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainhb.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainhc.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainhd.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainhe.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainhf.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainhg.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainhh.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainhi.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainhj.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainhk.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainhl.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainhm.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainhn.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainho.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainhp.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainhq.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainhr.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainhs.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainht.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainhu.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainhv.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainhw.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainhx.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainhy.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainhz.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traini-.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traini0.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traini1.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traini2.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traini3.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traini4.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traini5.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traini6.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traini7.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traini8.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traini9.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainiA.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainiB.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainiC.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainiD.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainiE.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainiF.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainiG.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainiH.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainiI.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainiJ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainiK.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainiL.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainiM.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainiN.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainiO.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainiP.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainiQ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainiR.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainiS.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainiT.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainiU.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainiV.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainiW.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainiX.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainiY.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainiZ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traini_.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainia.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainib.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainic.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainid.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainie.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainif.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainig.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainih.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainii.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainij.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainik.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainil.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainim.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainin.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainio.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainip.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainiq.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainir.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainis.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainit.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainiu.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainiv.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainiw.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainix.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainiy.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainiz.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainj-.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainj0.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainj1.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainj2.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainj3.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainj4.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainj5.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainj6.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainj7.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainj8.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainj9.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainjA.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainjB.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainjC.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainjD.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainjE.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainjF.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainjG.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainjH.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainjI.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainjJ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainjK.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainjL.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainjM.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainjN.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainjO.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainjP.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainjQ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainjR.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainjS.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainjT.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainjU.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainjV.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainjW.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainjX.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainjY.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainjZ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainj_.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainja.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainjb.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainjc.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainjd.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainje.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainjf.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainjg.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainjh.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainji.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainjj.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainjk.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainjl.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainjm.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainjn.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainjo.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainjp.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainjq.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainjr.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainjs.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainjt.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainju.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainjv.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainjw.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainjx.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainjy.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainjz.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traink-.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traink0.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traink1.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traink2.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traink3.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traink4.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traink5.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traink6.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traink7.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traink8.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traink9.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainkA.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainkB.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainkC.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainkD.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainkE.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainkF.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainkG.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainkH.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainkI.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainkJ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainkK.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainkL.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainkM.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainkN.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainkO.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainkP.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainkQ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainkR.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainkS.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainkT.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainkU.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainkV.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainkW.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainkX.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainkY.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainkZ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traink_.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainka.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainkb.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainkc.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainkd.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainke.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainkf.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainkg.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainkh.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainki.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainkj.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainkk.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainkl.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainkm.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainkn.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainko.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainkp.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainkq.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainkr.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainks.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainkt.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainku.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainkv.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainkw.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainkx.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainky.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainkz.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainl-.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainl0.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainl1.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainl2.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainl3.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainl4.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainl5.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainl6.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainl7.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainl8.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainl9.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainlA.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainlB.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainlC.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainlD.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainlE.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainlF.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainlG.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainlH.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainlI.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainlJ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainlK.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainlL.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainlM.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainlN.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainlO.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainlP.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainlQ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainlR.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainlS.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainlT.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainlU.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainlV.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainlW.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainlX.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainlY.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainlZ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainl_.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainla.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainlb.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainlc.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainld.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainle.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainlf.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainlg.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainlh.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainli.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainlj.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainlk.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainll.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainlm.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainln.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainlo.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainlp.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainlq.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainlr.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainls.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainlt.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainlu.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainlv.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainlw.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainlx.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainly.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainlz.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainm-.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainm0.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainm1.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainm2.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainm3.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainm4.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainm5.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainm6.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainm7.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainm8.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainm9.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainmA.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainmB.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainmC.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainmD.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainmE.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainmF.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainmG.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainmH.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainmI.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainmJ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainmK.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainmL.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainmM.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainmN.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainmO.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainmP.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainmQ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainmR.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainmS.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainmT.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainmU.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainmV.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainmW.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainmX.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainmY.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainmZ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainm_.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainma.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainmb.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainmc.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainmd.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainme.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainmf.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainmg.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainmh.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainmi.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainmj.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainmk.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainml.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainmm.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainmn.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainmo.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainmp.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainmq.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainmr.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainms.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainmt.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainmu.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainmv.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainmw.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainmx.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainmy.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainmz.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainn-.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainn0.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainn1.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainn2.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainn3.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainn4.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainn5.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainn6.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainn7.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainn8.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainn9.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainnA.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainnB.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainnC.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainnD.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainnE.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainnF.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainnG.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainnH.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainnI.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainnJ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainnK.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainnL.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainnM.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainnN.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainnO.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainnP.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainnQ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainnR.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainnS.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainnT.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainnU.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainnV.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainnW.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainnX.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainnY.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainnZ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainn_.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainna.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainnb.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainnc.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainnd.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainne.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainnf.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainng.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainnh.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainni.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainnj.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainnk.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainnl.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainnm.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainnn.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainno.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainnp.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainnq.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainnr.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainns.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainnt.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainnu.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainnv.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainnw.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainnx.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainny.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainnz.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traino-.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traino0.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traino1.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traino2.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traino3.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traino4.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traino5.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traino6.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traino7.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traino8.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traino9.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainoA.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainoB.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainoC.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainoD.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainoE.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainoF.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainoG.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainoH.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainoI.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainoJ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainoK.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainoL.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainoM.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainoN.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainoO.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainoP.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainoQ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainoR.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainoS.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainoT.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainoU.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainoV.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainoW.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainoX.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainoY.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainoZ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traino_.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainoa.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainob.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainoc.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainod.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainoe.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainof.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainog.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainoh.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainoi.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainoj.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainok.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainol.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainom.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainon.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainoo.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainop.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainoq.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainor.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainos.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainot.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainou.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainov.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainow.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainox.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainoy.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainoz.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainp-.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainp0.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainp1.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainp2.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainp3.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainp4.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainp5.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainp6.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainp7.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainp8.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainp9.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainpA.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainpB.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainpC.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainpD.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainpE.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainpF.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainpG.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainpH.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainpI.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainpJ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainpK.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainpL.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainpM.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainpN.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainpO.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainpP.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainpQ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainpR.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainpS.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainpT.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainpU.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainpV.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainpW.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainpX.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainpY.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainpZ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainp_.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainpa.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainpb.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainpc.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainpd.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainpe.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainpf.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainpg.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainph.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainpi.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainpj.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainpk.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainpl.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainpm.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainpn.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainpo.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainpp.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainpq.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainpr.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainps.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainpt.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainpu.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainpv.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainpw.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainpx.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainpy.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainpz.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainq-.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainq0.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainq1.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainq2.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainq3.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainq4.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainq5.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainq6.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainq7.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainq8.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainq9.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainqA.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainqB.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainqC.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainqD.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainqE.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainqF.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainqG.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainqH.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainqI.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainqJ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainqK.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainqL.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainqM.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainqN.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainqO.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainqP.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainqQ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainqR.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainqS.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainqT.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainqU.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainqV.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainqW.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainqX.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainqY.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainqZ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainq_.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainqa.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainqb.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainqc.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainqd.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainqe.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainqf.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainqg.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainqh.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainqi.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainqj.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainqk.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainql.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainqm.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainqn.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainqo.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainqp.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainqq.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainqr.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainqs.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainqt.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainqu.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainqv.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainqw.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainqx.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainqy.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainqz.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainr-.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainr0.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainr1.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainr2.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainr3.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainr4.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainr5.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainr6.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainr7.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainr8.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainr9.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainrA.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainrB.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainrC.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainrD.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainrE.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainrF.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainrG.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainrH.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainrI.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainrJ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainrK.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainrL.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainrM.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainrN.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainrO.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainrP.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainrQ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainrR.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainrS.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainrT.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainrU.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainrV.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainrW.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainrX.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainrY.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainrZ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainr_.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainra.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainrb.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainrc.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainrd.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainre.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainrf.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainrg.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainrh.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainri.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainrj.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainrk.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainrl.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainrm.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainrn.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainro.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainrp.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainrq.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainrr.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainrs.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainrt.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainru.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainrv.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainrw.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainrx.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainry.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainrz.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trains-.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trains0.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trains1.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trains2.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trains3.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trains4.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trains5.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trains6.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trains7.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trains8.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trains9.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainsA.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainsB.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainsC.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainsD.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainsE.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainsF.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainsG.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainsH.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainsI.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainsJ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainsK.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainsL.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainsM.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainsN.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainsO.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainsP.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainsQ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainsR.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainsS.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainsT.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainsU.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainsV.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainsW.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainsX.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainsY.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainsZ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trains_.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainsa.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainsb.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainsc.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainsd.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainse.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainsf.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainsg.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainsh.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainsi.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainsj.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainsk.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainsl.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainsm.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainsn.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainso.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainsp.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainsq.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainsr.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainss.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainst.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainsu.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainsv.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainsw.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainsx.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainsy.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainsz.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traint-.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traint0.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traint1.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traint2.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traint3.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traint4.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traint5.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traint6.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traint7.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traint8.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traint9.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traintA.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traintB.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traintC.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traintD.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traintE.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traintF.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traintG.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traintH.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traintI.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traintJ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traintK.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traintL.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traintM.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traintN.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traintO.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traintP.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traintQ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traintR.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traintS.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traintT.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traintU.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traintV.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traintW.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traintX.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traintY.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traintZ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traint_.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainta.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traintb.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traintc.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traintd.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainte.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traintf.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traintg.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainth.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainti.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traintj.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traintk.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traintl.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traintm.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traintn.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainto.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traintp.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traintq.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traintr.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traints.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traintt.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traintu.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traintv.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traintw.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traintx.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainty.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/traintz.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainu-.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainu0.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainu1.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainu2.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainu3.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainu4.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainu5.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainu6.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainu7.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainu8.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainu9.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainuA.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainuB.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainuC.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainuD.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainuE.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainuF.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainuG.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainuH.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainuI.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainuJ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainuK.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainuL.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainuM.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainuN.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainuO.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainuP.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainuQ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainuR.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainuS.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainuT.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainuU.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainuV.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainuW.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainuX.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainuY.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainuZ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainu_.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainua.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainub.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainuc.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainud.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainue.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainuf.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainug.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainuh.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainui.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainuj.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainuk.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainul.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainum.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainun.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainuo.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainup.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainuq.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainur.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainus.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainut.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainuu.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainuv.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainuw.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainux.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainuy.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainuz.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainv-.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainv0.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainv1.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainv2.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainv3.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainv4.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainv5.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainv6.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainv7.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainv8.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainv9.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainvA.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainvB.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainvC.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainvD.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainvE.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainvF.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainvG.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainvH.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainvI.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainvJ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainvK.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainvL.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainvM.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainvN.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainvO.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainvP.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainvQ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainvR.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainvS.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainvT.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainvU.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainvV.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainvW.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainvX.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainvY.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainvZ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainv_.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainva.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainvb.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainvc.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainvd.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainve.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainvf.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainvg.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainvh.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainvi.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainvj.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainvk.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainvl.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainvm.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainvn.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainvo.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainvp.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainvq.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainvr.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainvs.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainvt.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainvu.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainvv.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainvw.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainvx.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainvy.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainvz.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainw-.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainw0.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainw1.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainw2.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainw3.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainw4.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainw5.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainw6.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainw7.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainw8.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainw9.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainwA.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainwB.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainwC.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainwD.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainwE.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainwF.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainwG.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainwH.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainwI.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainwJ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainwK.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainwL.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainwM.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainwN.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainwO.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainwP.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainwQ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainwR.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainwS.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainwT.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainwU.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainwV.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainwW.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainwX.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainwY.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainwZ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainw_.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainwa.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainwb.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainwc.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainwd.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainwe.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainwf.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainwg.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainwh.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainwi.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainwj.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainwk.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainwl.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainwm.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainwn.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainwo.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainwp.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainwq.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainwr.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainws.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainwt.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainwu.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainwv.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainww.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainwx.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainwy.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainwz.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainx-.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainx0.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainx1.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainx2.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainx3.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainx4.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainx5.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainx6.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainx7.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainx8.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainx9.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainxA.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainxB.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainxC.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainxD.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainxE.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainxF.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainxG.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainxH.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainxI.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainxJ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainxK.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainxL.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainxM.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainxN.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainxO.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainxP.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainxQ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainxR.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainxS.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainxT.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainxU.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainxV.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainxW.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainxX.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainxY.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainxZ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainx_.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainxa.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainxb.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainxc.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainxd.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainxe.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainxf.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainxg.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainxh.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainxi.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainxj.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainxk.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainxl.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainxm.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainxn.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainxo.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainxp.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainxq.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainxr.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainxs.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainxt.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainxu.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainxv.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainxw.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainxx.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainxy.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainxz.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainy-.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainy0.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainy1.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainy2.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainy3.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainy4.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainy5.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainy6.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainy7.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainy8.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainy9.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainyA.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainyB.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainyC.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainyD.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainyE.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainyF.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainyG.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainyH.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainyI.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainyJ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainyK.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainyL.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainyM.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainyN.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainyO.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainyP.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainyQ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainyR.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainyS.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainyT.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainyU.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainyV.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainyW.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainyX.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainyY.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainyZ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainy_.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainya.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainyb.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainyc.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainyd.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainye.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainyf.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainyg.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainyh.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainyi.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainyj.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainyk.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainyl.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainym.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainyn.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainyo.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainyp.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainyq.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainyr.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainys.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainyt.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainyu.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainyv.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainyw.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainyx.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainyy.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainyz.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainz-.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainz0.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainz1.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainz2.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainz3.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainz4.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainz5.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainz6.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainz7.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainz8.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainz9.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainzA.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainzB.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainzC.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainzD.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainzE.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainzF.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainzG.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainzH.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainzI.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainzJ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainzK.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainzL.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainzM.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainzN.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainzO.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainzP.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainzQ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainzR.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainzS.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainzT.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainzU.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainzV.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainzW.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainzX.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainzY.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainzZ.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainz_.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainza.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainzb.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainzc.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainzd.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainze.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainzf.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainzg.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainzh.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainzi.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainzj.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainzk.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainzl.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainzm.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainzn.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainzo.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainzp.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainzq.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainzr.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainzs.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainzt.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainzu.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainzv.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainzw.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainzx.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainzy.tfrecord', 'gs://alassane-ndiaye-regional/youtube-8m/video-level-train/trainzz.tfrecord']
    data = File_Queue(record_list, feature_map, preprocess,
                      inputs_dim=inputs_dim, labels_dim=labels_dim, batch_size=1000)
    make_conv_network(inputs_dim, labels_dim, data)
    iterator = launch_training_task(data, use_queue=True)
    while True:
        print(next(iterator))
